import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
/**
 * Example service implementation for pushing notifications to client applications using HTTP POST method of Notifications API.
 */
public class MyService {
    private final static String REST_API_BASE_URI = "https://alpha.one.ovi.com/nnapi/1.0/";

    private final static String MANDATORY_PARAMETER_MISSING = "Mandatory notification parameter is missing. Mandatory parameters are: service_id, service_secret, payload and nid.\n" +
            "Parameters must be given in format: PARAMETER1:VALUE1 PARAMETER2:VALUE2";

    private DefaultHttpClient client;
    private final String serviceId;
    private final String serviceSecret;

    /**
     * Constructor for the service class.
     *
     * @param serviceId Notifications API service id
     * @param serviceSecret Notifications API service secret
     * @throws Exception exception if there was problems in arguments or notification sending,
     */
    public MyService(String serviceId, String serviceSecret) throws Exception {
        this.serviceId = serviceId;
        this.serviceSecret = serviceSecret;

        initHttpsClient();
    }

    /**
     * Main method for MyService. Creates a MyService object instance and pushes notifications with given payload to the client application.
     *
     * @param args arguments for pushing the notifications to applications. Arguments must be in the form of ARGUMENT:VALUE
     */
    public static void main(String[] args) {
        try {
            //Arguments must be given to push the notifications.
            if (args.length == 0) {
                throw new IllegalArgumentException(MANDATORY_PARAMETER_MISSING);
            }

            //Parse command line arguments
            Map<String, String> parameters = parseArguments(args);

            //Create service instance
            MyService service = new MyService(parameters.get("service_id"), parameters.get("service_secret"));

            //Push the notifications to the client application. By default one notification is sent but with count parameter the amount can be configured.
            int notificationAmount = parameters.get("count") == null ? 1 : Integer.valueOf(parameters.get("count"));
            for (int sent = 0; sent < notificationAmount; sent++) {
                service.sendNotificationWithNotificationId(parameters.get("payload"), parameters.get("nid"));
            }

            service.shutdownConnection();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        System.exit(0);
    }

    /**
     * Parses the argument list to a map containing the parameter type and value containing the notification recipient
     * and payload.
     *
     * @param args array of notification arguments.
     * @return parameters parsed to a Map instance with parameter name as key and the given value assigned to that key.
     * @throws IllegalArgumentException if required arguments are not set.
     */
    private static Map<String, String> parseArguments(String[] args) throws Exception {
        Map<String, String> parameters = new HashMap<String, String>();
for (String arg : args) {
            String[] parameter = arg.split(":");
            if (parameter.length != 2) {
                throw new IllegalArgumentException("Argument in wrong format: " + arg);
            }
            parameters.put(parameter[0].toLowerCase(), parameter[1]);
        }

        //Check that the required arguments are found.
        if (!parameters.containsKey("payload") || !parameters.containsKey("service_id") || !parameters.containsKey("service_secret") || !parameters.containsKey("nid")) {
            throw new IllegalArgumentException(MANDATORY_PARAMETER_MISSING);
        }

        return parameters;
    }

    /**
     * Creates the HTTPS client that is used for executing the HTTP requests.
     */
    private void initHttpsClient() {
        Scheme httpScheme = new Scheme("https", SSLSocketFactory.getSocketFactory(), 443);
        SchemeRegistry sr = new SchemeRegistry();
        sr.register(httpScheme);
        HttpParams params = new BasicHttpParams();
        ClientConnectionManager cm = new ThreadSafeClientConnManager(params, sr);
        client = new DefaultHttpClient(cm, params);

        Credentials creds = new UsernamePasswordCredentials(serviceId, serviceSecret);
        client.getCredentialsProvider().setCredentials(new AuthScope("alpha.one.ovi.com", 443, AuthScope.ANY_REALM), creds);
    }

    /**
     * Pushes notification to the client application using the Notification ID REST API functionality.
     * @param payload notification payload.
     * @param nid notification id containing the recipient client and application information.
     */
    private void sendNotificationWithNotificationId(String payload, String nid) {
        //Notification ID must be URL encoded
        String encodedNid;
        try {
            encodedNid = URLEncoder.encode(nid, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Notification ID URL encoding failed:" + e.getMessage());
        }

        List<NameValuePair> headerParameters = new ArrayList<NameValuePair>();
        headerParameters.add(new BasicNameValuePair("payload", payload));

        sendHttpPost(headerParameters, REST_API_BASE_URI + "nid/" + encodedNid);
    }

    /**
     * Executes the HTTP POST request to the Notifications API.
     * @param headerParameters header parameter for the POST request.
     * @param requestUri uri for the request.
     */
    private void sendHttpPost(List<NameValuePair> headerParameters, String requestUri) {
        //Create the HTTP POST Request
        HttpPost httpPost = new HttpPost(requestUri.toString());
        try {
            httpPost.setEntity(new UrlEncodedFormEntity(headerParameters, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Encoding error while converting HTTP parameters to UTF-8.", e);
        }
        //Push the notification with HTTPS client
        try {
            HttpResponse response = client.execute(httpPost);
            System.out.println("HTTP Response from Notifications API: " + response.getStatusLine().getStatusCode());
            if (response.getEntity() != null) {
                response.getEntity().consumeContent();
            }
        } catch (IOException e) {
            throw new RuntimeException("HTTP POST Request to " + requestUri.toString() + " failed with error: " + e.getMessage());
        }
    }

    /**
     * Closes the connection properly.
     */
    private void shutdownConnection() {
        client.getConnectionManager().shutdown();
    }

}